!-------------------------------------------------------------------------------

!                      Code_Saturne version 4.0.1
!                      --------------------------
! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2015 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

!===============================================================================
! Purpose:
! -------

!> \file cs_user_extra_operations.f90
!>
!> \brief This function is called at the end of each time step, and has a very
!>  general purpose
!>  (i.e. anything that does not have another dedicated user subroutine)
!>
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]     nvar          total number of variables
!> \param[in]     nscal         total number of scalars
!> \param[in]     dt            time step (per cell)
!_______________________________________________________________________________

subroutine cs_f_user_extra_operations &
 ( nvar   , nscal  ,                                              &
   dt     )

!===============================================================================

!===============================================================================
! Module files
!===============================================================================

use paramx
use dimens, only: ndimfb
use pointe
use numvar
use optcal
use cstphy
use cstnum
use entsor
use lagpar
use lagran
use lagdim
use parall
use period
use ppppar
use ppthch
use ppincl
use mesh
use field
use field_operator
use cs_c_bindings
use cs_f_interfaces

!===============================================================================

implicit none

! Arguments

integer          nvar   , nscal

double precision dt(ncelet)

! Local variables

integer id_name
integer ifac, iel
integer inc, iprev
double precision, dimension(:,:), pointer :: tau
double precision, dimension(:,:,:), allocatable :: gradv
double precision norm(3)
double precision dudx, dvdx, dwdx, dudy, dvdy, dwdy, dudz, dvdz, dwdz
double precision mu
double precision, dimension(:), pointer :: viscl, cvisct

!< [loc_var_dec]
integer          ii
integer          ilelt  , nlelt
integer          ioutput
!double precision filenumber

double precision xfor(3)
double precision a1, a2
double precision, dimension(:,:), pointer :: bfprp_for

double precision AoA, V, L, D, Cl, Cd

integer, allocatable, dimension(:) :: lstelt
!< [loc_var_dec]


!===============================================================================

!===============================================================================
! Initialization
!===============================================================================

if (ineedf.eq.1) call field_get_val_v(iforbr, bfprp_for)

! Allocate a temporary array for cells or interior/boundary faces selection
allocate(lstelt(max(ncel,nfac,nfabor)))

!!! Output Force on the aerofoil:

if (ineedf.eq.1) then

  do ii = 1, ndim
    xfor(ii) = 0.d0
  enddo

  call getfbr('AIRFOIL', nlelt, lstelt)
  !==========

  do ilelt = 1, nlelt

    ifac = lstelt(ilelt)

    do ii = 1, ndim
      xfor(ii) = xfor(ii) + bfprp_for(ii, ifac)
    enddo

  enddo

  if (irangp.ge.0) then
    call parrsm(ndim,xfor)
  endif
  
  AoA = 0.0   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!change AOA!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  V = 1.45358  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!change V !!!!!!!!!!!!!!!!!!!!!!!!!!
  if (irangp.le.0) then
    L = -xfor(1)*sin(AoA/180.*3.14159265359) + xfor(2)*cos(AoA/180.*3.14159265359)
    D = xfor(1)*cos(AoA/180.*3.14159265359) + xfor(2)*sin(AoA/180.*3.14159265359)
    Cl = L / 0.5 / 1.225 / V / V / 0.1
    Cd = D / 0.5 / 1.225 / V / V / 0.1
    !write(nfecra,*)xfor(1),xfor(2),xfor(3)
    write(nfecra,*) Cl, Cd, xfor(3)
  endif
  
!Output Forces
  if (irangp.le.0) then

    !write(impusr(1),*)ntcabs,xfor(1),xfor(2),xfor(3)
    write(impusr(1),*)ntcabs, Cl, Cd, xfor(3)
  endif
 
endif


!!!!Output Force in different z positions on the aerofoil:

if (ineedf.eq.1) then

  do ioutput = 2, 21      !!!!!!!!!!!!!change:NUMBER OF SPANWISE NODES!!!!!!!!!!!!!!!!

      do ii = 1, ndim
        xfor(ii) = 0.d0
      enddo
  
      a1 = 0.1d0 * dble(ioutput-2) / 20.0d0     !!!!!!!!!!!!!change:NUMBER OF SPANWISE ELEMENTS!!!!!!!!!!!!!!
      a2 = 0.1d0 * dble(ioutput-1) / 20.0d0     !!!!!!!!!!!!!change:NUMBER OF SPANWISE ELEMENTS!!!!!!!!!!!!!!

     ! write(nfecra,*) a1, a2
      call getfbr('AIRFOIL', nlelt, lstelt)           
  !==========

      do ilelt = 1, nlelt

        ifac = lstelt(ilelt)
        
        if ((cdgfbo(3,ifac) .gt. a1) .and. (cdgfbo(3,ifac) .lt. a2)) then

          do ii = 1, ndim
            xfor(ii) = xfor(ii) + bfprp_for(ii, ifac)
          enddo
        
        endif

      enddo

       if (irangp.ge.0) then
         call parrsm(ndim,xfor)
       endif
  
        if (irangp.le.0) then
          L = -xfor(1)*sin(AoA/180.*3.14159265359) + xfor(2)*cos(AoA/180.*3.14159265359)
          D = xfor(1)*cos(AoA/180.*3.14159265359) + xfor(2)*sin(AoA/180.*3.14159265359)
          Cl = L / 0.5 / 1.225 / V / V / 0.1
          Cd = D / 0.5 / 1.225 / V / V / 0.1
          write(impusr(ioutput),*)ntcabs, Cl, Cd, xfor(3)
          !write(impusr(ioutput),*)ntcabs,xfor(1),xfor(2),xfor(3)

        endif
  
  enddo
 
endif


! Deallocate the temporary array
deallocate(lstelt)

!===============================================================================
! User operations
!===============================================================================

allocate(gradv(3,3,ncelet))

!call field_get_id('wall_shear',id_name)
call field_get_id_try('wall_shear',id_name)
call field_get_val_v(id_name, tau)



call field_get_val_s(iprpfl(ivisct), cvisct)
call field_get_val_s(iprpfl(iviscl), viscl)

! Velocity gradient
inc = 1
iprev = 1

call field_gradient_vector(ivarfl(iu), iprev, imrgra, inc,   &
                           gradv)

do ifac = 1, nfabor

   iel = ifabor(ifac)

   mu = cvisct(iel) + viscl(iel)

   norm(1) = surfbo(1,ifac) / surfbn(ifac)
   norm(2) = surfbo(2,ifac) / surfbn(ifac)
   norm(3) = surfbo(3,ifac) / surfbn(ifac)

   dudx = gradv(1,1,iel)
   dvdx = gradv(1,2,iel)
   dwdx = gradv(1,3,iel)
   dudy = gradv(2,1,iel)
   dvdy = gradv(2,2,iel)
   dwdy = gradv(2,3,iel)
   dudz = gradv(3,1,iel)
   dvdz = gradv(3,2,iel)
   dwdz = gradv(3,3,iel)

   tau(ifac,1) = mu*(norm(1)*2.d0*dudx + norm(2)*(dvdx+dudy) + norm(3)*(dwdx+dudz))
   tau(ifac,2) = mu*(norm(1)*(dudy+dvdx) + norm(2)*2.d0*dvdy + norm(3)*(dwdy+dvdz))
   tau(ifac,3) = mu*(norm(2)*(dudz+dwdx) + norm(2)*(dvdz+dwdy) + norm(3)*2.d0*dwdz)

enddo


!----
! End
!----

deallocate(gradv)

return
end subroutine cs_f_user_extra_operations
