/*============================================================================
 * User subroutines for input of calculation parameters.
 *============================================================================*/

/* Code_Saturne version 4.0.1 */

/*
  This file is part of Code_Saturne, a general-purpose CFD tool.

  Copyright (C) 1998-2015 EDF S.A.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
  Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

/*----------------------------------------------------------------------------*/

#include "cs_defs.h"

/*----------------------------------------------------------------------------
 * Standard C library headers
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <math.h>
#include <string.h>

#if defined(HAVE_MPI)
#include <mpi.h>
#endif

/*----------------------------------------------------------------------------
 * PLE library headers
 *----------------------------------------------------------------------------*/

#include <ple_coupling.h>

/*----------------------------------------------------------------------------
 *  Local headers
 *----------------------------------------------------------------------------*/

#include "bft_mem.h"
#include "bft_error.h"
#include "bft_printf.h"

#include "fvm_writer.h"

#include "cs_base.h"
#include "cs_field.h"
#include "cs_gui_util.h"
#include "cs_field_pointer.h"
#include "cs_field_operator.h"
#include "cs_math.h"
#include "cs_mesh.h"
#include "cs_mesh_location.h"
#include "cs_mesh_quantities.h"
#include "cs_halo.h"
#include "cs_halo_perio.h"
#include "cs_log.h"
#include "cs_multigrid.h"
#include "cs_parameters.h"
#include "cs_physical_constants.h"
#include "cs_prototypes.h"
#include "cs_rotation.h"
#include "cs_sles.h"
#include "cs_sles_it.h"
#include "cs_time_moment.h"
#include "cs_time_step.h"
#include "cs_turbomachinery.h"
#include "cs_selector.h"

#include "cs_post.h"

/*----------------------------------------------------------------------------
 *  Header for the current file
 *----------------------------------------------------------------------------*/

#include "cs_prototypes.h"

/*----------------------------------------------------------------------------*/

BEGIN_C_DECLS

/*============================================================================
 * User function definitions
 *============================================================================*/


void
cs_user_time_moments(void)
{

/* tau_x time moment*/
{
const cs_field_t *var = cs_field_by_name_try("wall_shear");

int moment_f_id[] = {var->id};
int moment_c_id[] = {0};
int n_fields = 1;

cs_time_moment_define_by_field_ids("tau_x",
                         n_fields,
                         moment_f_id,
                         moment_c_id,
                         CS_TIME_MOMENT_MEAN,
                         380001, /* Time step start*/
                         -1,
                         CS_TIME_MOMENT_RESTART_AUTO,
                         NULL);
}


/* tau_y time moment*/
{
const cs_field_t *var = cs_field_by_name_try("wall_shear");

int moment_f_id[] = {var->id};
int moment_c_id[] = {1};
int n_fields = 1;

cs_time_moment_define_by_field_ids("tau_y",
                         n_fields,
                         moment_f_id,
                         moment_c_id,
                         CS_TIME_MOMENT_MEAN,
                         380001, /* Time step start*/
                         -1,
                         CS_TIME_MOMENT_RESTART_AUTO,
                         NULL);
}

/* tau_z time moment*/
{
const cs_field_t *var = cs_field_by_name_try("wall_shear");

int moment_f_id[] = {var->id};
int moment_c_id[] = {2};
int n_fields = 1;

cs_time_moment_define_by_field_ids("tau_z",
                         n_fields,
                         moment_f_id,
                         moment_c_id,
                         CS_TIME_MOMENT_MEAN,
                         380001, /* Time step start*/
                         -1,
                         CS_TIME_MOMENT_RESTART_AUTO,
                         NULL);
}


}

END_C_DECLS
